%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dynare file for a basic New Keynesian Model
%
% See Gali's textbook, "Monetary Policy, Inflation, and the Business Cycle"
% (2008) for details of the model in chapter 3. It is augmented to include
% cost push and aggregate demand shocks.
%
% The model is simulated in order to describe the evolution of deviations 
% from the steady-state of endogenous variables following a shock
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

var pi          % inflation
    y_gap       % output gap
    y_ss       	% steady-state of output
    y           % output
    r_nat       % natural interest rate
    r_real      % real interest rate
    in          % nominal interst rate
    n           % hours worked
    m_grow_ann	% money growth
    nu          % AR(1) monetary policy shock process
    a           % AR(1) technology shock process
    pssi        % AR(1) cost-push process
    gammma      % AR(1) aggregate demand process
    r_real_ann  % annualized real interest rate
    i_ann       % annualized nominal interest rate
    r_nat_ann   % annualized natural interest rate
    pi_ann;     % annualized inflation rate

varexo eps_a    % technology shock
       eps_nu   % monetary policy shock
       eps_c    % cost-push shock
       eps_d;   % aggregate demand shock

parameters sigmma phi phi_pi phi_y theta rho_nu rho_a rho_c rho_d betta eta alphha epsilon Omega psi_n_ya lambda kappa;

sigmma = 1;     % log utility
phi = 1;        % unitary Frisch elasticity
%phi_pi = 1.5;   % inflation feedback Taylor Rule
%phi_y = .5/4;   % output feedback Taylor Rule
theta = 2/3;    % Calvo parameter
%rho_nu = 0.5;   % autocorrelation monetary policy shock
rho_a  = 0.9;   % autocorrelation technology shock
%rho_c  = 0.5;   % autocorrelation cost-push shock
%rho_d  = 0.5;   % autocorrelation demand shock
betta = 0.99;   % discount factor
eta = 4;        % semi-elasticity of money demand
alphha = 1/3;   % capital share
epsilon = 6;    % demand elasticity
Omega = (1-alphha)/(1-alphha+alphha*epsilon);      % see pg 47
psi_n_ya = (1+phi)/(sigmma*(1-alphha)+phi+alphha); % see pg 48
lambda = (1-theta)*(1-betta*theta)/theta*Omega;    % see pg 47
kappa = lambda*(sigmma+(phi+alphha)/(1-alphha));   % see pg 49

model(linear); 
pi=betta*pi(+1)+kappa*y_gap+pssi;			% 1 NK Phillips Curve (eq. 21)
y_gap=-1/sigmma*(in-pi(+1)-r_nat)+y_gap(+1)+gammma;	% 2 Dyn IS Curve (eq. 22)
in=phi_pi*pi+phi_y*y_gap+nu;			% 3 Interest Rate Rule (eq. 25)
y=a+(1-alphha)*n;                       % 4 Production function (eq. 13)
m_grow_ann=4*(y-y(-1)-eta*(in-in(-1))+pi);	% 5 Money growth (eq. (4)
r_nat=sigmma*psi_n_ya*(a(+1)-a);		% 6 Natural interest (eq. 23)
r_real=in-pi(+1);                       % 7 Real interest rate
y_ss=phi*a;                             % 8 Steady-state of output
y_gap=y-y_ss;                           % 9 Output gap
i_ann=4*in;                             % 10 Annual nom interest rate
r_real_ann=4*r_real;                    % 11 Annual real interest rate
r_nat_ann=4*r_nat;                      % 14 Annual nat interest rate
pi_ann=4*pi;                            % 15 Annual inflation
nu=rho_nu*nu(-1)+eps_nu;                % 16 Monetary policy shock
a=rho_a*a(-1)+eps_a;                    % 17 Technology shock
pssi=rho_c*pssi(-1)+eps_c;              % 18 Cost-push shock
gammma=rho_d*gammma(-1)+eps_d;          % 19 Aggregate demand shock
end;

varobs y_gap pi in; 

estimated_params; 
phi, gamma_pdf, 1, 0.05;
theta, beta_pdf, 0.66, 0.1;
rho_nu, gamma_pdf, 0.5, 0.05;
rho_c, gamma_pdf, 0.5, 0.1;
rho_d, gamma_pdf, 0.5, 0.1;
phi_y, gamma_pdf, 0.25, 0.05;
phi_pi, gamma_pdf, 1.5, 0.05;
stderr eps_nu, inv_gamma_pdf, 0.005, inf; 
stderr eps_c, inv_gamma_pdf, 0.005, inf; 
stderr eps_d, inv_gamma_pdf, 0.005, inf; 
end;

shocks;
var eps_a  = 1^2;       % 1 std dev shock to technology
end;

//estimation(datafile='SAdata.xls',prefilter=1,mh_replic=2000,mh_nblocks=1,mh_drop=0.45);
estimation(datafile='SAdata.xls',order=2,mh_replic=2000,mode_compute=8,filter_algorithm=sis);

stoch_simul(order = 1,irf=15) y_gap pi_ann y n i_ann r_real_ann m_grow_ann nu a;
